import marimo

__generated_with = "0.6.17"
app = marimo.App(width="medium")


@app.cell
def __():
    import cvxpy as cp
    import marimo as mo
    return cp, mo


@app.cell(hide_code=True)
def __(mo):
    mo.md("""
    ## Exercise 2 and 4
    """)
    return


@app.cell
def __(cp):
    x = cp.Variable()
    y = cp.Variable()
    z = cp.Variable()

    # a.

    _f = cp.sqrt(1 + 4 * cp.square(x) + 16 * cp.square(y))

    # not DCP:
    assert not _f.is_dcp()

    # DCP-compliant rewrite:
    _f = cp.norm2(cp.hstack([1, 2*x, 4*y]))
    assert _f.is_dcp()

    # b.
    _g = cp.minimum(x, cp.log(y)) - cp.maximum(y, z)

    # already DCP:
    assert _g.is_dcp()

    # c.

    _h = cp.log(cp.exp(2*x + 3) + cp.exp(4 * y + 5))

    # not DCP:
    assert not _h.is_dcp()

    # DCP-compliant rewrite:
    _h = cp.log_sum_exp(cp.hstack([2 * x + 3, 4 * y + 5]))
    assert _h.is_dcp()
    return x, y, z


@app.cell(hide_code=True)
def __(mo):
    mo.md("""
    ## Exercise 5

    Since $g$ is the total fuel in kg when moving a distance $d$ in time $t$, we have $g(d,t) = t f(d/t)$.
    The function is known as the perspective function of $f$, and is convex when $f$ is convex (see Boyd and Vandenberghe, §3.2.6).
    """)
    return


@app.cell
def __(cp, x):
    # f can by any DCP-compliant function, we use a simple one for convenience
    _f = cp.square(x)
    assert _f.is_dcp()

    # d has to be nonnegative
    d = cp.Variable(nonneg=True)

    _g = cp.perspective(_f, d)
    assert _g.is_dcp()
    return d,


if __name__ == "__main__":
    app.run()
